
// -------------------------------------------------------------------------------------
// This class uses the code from "MSCP - Marcel's Simple Chess Program", adapted by miko. 
// MSCP: marcelk@bitpit.net / Marcel van Kervinck / http://marcelk.net/mscp/
// August 2010
// -------------------------------------------------------------------------------------

#pragma once

	struct side
	{           
        byte attack[64];
        int king;
        byte pawns[10];
	};

	struct move
	{
        short imove;
        unsigned short prescore;
	};

	struct command
	{
        char *name;
        void (*cmd)(char*);
        char *help;
	};

	/* Transposition table and opening book share the same memory */
	union TTBK
	{
        struct TT 
		{                     /* Transposition table entry */
                unsigned short hash;    /* - Identifies position */ 
                short move;             /* - Best recorded move */
                short score;            /* - Score */
                char flag;              /* - How to interpret score */
                char depth;             /* - Remaining search depth */
        } tt[CORE];
	};

	// Forward declarations
	DWORD WINAPI MyThreadFunction1(LPVOID lpParam);
	DWORD WINAPI MyThreadFunction2(LPVOID lpParam);


// ---------------------------------------------------------------------------------
class CMSCP
// ---------------------------------------------------------------------------------
{
public:
	CMSCP(void);
	~CMSCP(void);

	static const byte king_dirs[64];
	static const byte knight_dirs[64];
	static const int prescore_piece_value[13];
	static const int center[64];
	static const int pawn_advance[64];
	static const int white_control[64];

	static int __cdecl cmp_move(const void *ap, const void *bp);

	TTBK core;

	bool NewGame();
	std::string Update1();
	std::string Update2();
	void SetMaxSearchDepth(int iDepth);
	bool Go(std::wstring wcsCallbackFunction);
	CHESSCOLOR GetCurrentPlayer(void);
	bool ExecuteMove(char* pMove);
	int GetMaxSearchDepth();
	std::string GetValidMoves(char* pFrom);
	bool SetPositionsFEN(char* pPositions);
	std::string GetPositionsFEN(void);
	void SetCallbackFunction(std::string sResult);
	int GetCalcRunning();
	void CloseWorkerThread(void);
	bool AbortThinking();
	bool IsValidMove(char* pMove);
	bool IsKingAttacked();
	int GetNumberOfPossibleMoves(void);
	bool GoProposal(std::wstring wcsCallbackFunction);

private:

	byte board[64+3];		/* Board state */
	int ply;				/* Number of half-moves made in game and search */
	byte castle[64];		/* Which pieces may participate in castling */
	int computer[2];		/* Which side is played by the computer */
	unsigned long nodes;	/* Node counter */	
	struct side white, black, *myfriend, *enemy;
	unsigned short history[64*64];				/* History-move heuristic counters */
	signed char undo_stack[6*4096], *undo_sp;	/* Move undo administration. Was: 6*1024 miko */
	unsigned long hash_stack[4096];				/* History of hashes, for repetition Was: 1024 miko */
	int maxdepth;								/* Maximum search depth */
	struct move move_stack[4096], *move_sp;		/* History of moves was: 1024 = 128*8. Changed to 128*32 /miko */
	int piece_square[12][64];					/* Position evaluation tables */
	unsigned long zobrist[12][64];				/* Hash-key construction */
	signed char king_step[129];					/* Offsets for king moves */
	signed char knight_jump[129];				/* Offsets for knight jumps */ 
	long rnd_seed;
	unsigned short caps;

	long rnd(void);
	unsigned long compute_hash(void);
	void reset(void);
	void atk_slide(int sq, byte dirs, struct side *s);
	void compute_attacks(void);
	void unmake_move(void);
	void make_move(int move);
	int push_move(int fr, int to);
	void push_special_move(int fr, int to);
	void push_pawn_move(int fr, int to);
	void gen_slides(int fr, byte dirs);
	int test_illegal(int move);
	void generate_moves(unsigned treshold);
	int parse_move(char *line, int *num);
	std::string print_square(int square);
	std::string print_move_long(int move);
	bool setup_board(char *fen);
	void compute_piece_square_tables(void);
	int evaluate(void);
	int qsearch(int alpha, int beta);
	int search(int depth, int alpha, int beta);
	unsigned short squeeze(unsigned long n);
	int root_search(int maxdepth);
	void ComputerPlaysBoth();
	void ComputerPlaysNone();
	void ComputerPlaysWhite();
	void ComputerPlaysBlack();
	bool CheckForDrawByRepetition(void);
		
	bool m_bIsCalculating;
	bool m_bAbortCalculation;
	std::wstring m_wcsCallbackFunction;
	HANDLE m_hThreadHandle;
};
