//
// plugin.cpp - Standard plugin function definitions.  Hook into these functions to init/display/destroy your plugin.
//
// *** The names and syntax of these functions must remain the same, but you are free to insert your own code into each function body ***
// Please see 'readme.txt' for more information.

#include "stdafx.h"

#pragma warning(disable:4100)

#define PLUG_API __declspec(dllexport)

// Exported Plug-In methods
extern "C" PLUG_API BOOL GetDLLInfo(WCHAR* xml, DWORD dwMaxLength); // Returns XML describing the DLL
extern "C" PLUG_API BOOL Init(SPlayerInfo* info, DWORD* instancehandle); // Initializes the plugin with the information from the player
extern "C" PLUG_API BOOL Destroy(DWORD instancehandle); // Called when the plugin should release its resources
extern "C" PLUG_API BOOL DeviceLost(DWORD instancehandle); // Called when the DirectX device is lost (such as on a resize)
extern "C" PLUG_API BOOL DeviceRestore(DWORD instancehandle); // Called when the DirectX device is restored after a loss
extern "C" PLUG_API BOOL DocumentLoaded(DWORD instancehandle); // Called when the dxstudio document has finished loading
extern "C" PLUG_API BOOL EngineStage(DWORD instancehandle, ES stage, int pass, SEngineInfo* ei); // Called as requested by 'notify' tags.
extern "C" PLUG_API BOOL Update(DWORD instancehandle, FLOAT Seconds); // Called once per frame
extern "C" PLUG_API BOOL GetJSClass(DWORD instancehandle, JSClass** ppJSClass, JSPropertySpec** ppJSProps, JSFunctionSpec** ppJSMethods, void** ppPrivate); // Called if the player wants a JSObject for this plugin

SPlayerInfo* g_pInfo = NULL;
CMyClass myclass;
CMSCP mychess;

// Returns XML describing the DLL - v4 uses WCHAR*
// ---------------------------------------------------------------------------------
PLUG_API BOOL GetDLLInfo(WCHAR* xml, DWORD dwMaxLength)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"GetDLLInfo called.");
	
	// Advanced - a multidevice plugin can be defined here if required.  These plugins support multiple devices
	// at the same time (e.g. for multi windows), so init, render and destroy will be called with a different deviceindex in g_pInfo.
	// This means you need to load and maintain DirectX resources per-device.  This is more complex than
	// a normal plugin, so it defaults to multidevice='no'.
	const WCHAR* info = L"<dxpp version='4.1.0.0' multidevice='no'/>";

	if (dwMaxLength>wcslen(info))
		wcscpy(xml, info);
	else
		return FALSE;

	return TRUE;
}

// Initializes the plugin with the information from the player
// ---------------------------------------------------------------------------------
PLUG_API BOOL Init(SPlayerInfo* info, DWORD* instancehandleout)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"Init called.");

	#ifdef _DEBUG
		GL_ActivateMemoryDebugging();
	#endif
	
	// Not used for this sample, but you can use this as you like to have multiple instances.
	// Just create a multiple instances of CMyClass and assign a DWORD tag here.

	*instancehandleout = 0;
	g_pInfo=info;

	return TRUE;
}

// Called by the player if it wants a JSObject for this effect (accesed via script)
// ---------------------------------------------------------------------------------
PLUG_API BOOL GetJSClass(DWORD instancehandle, JSClass** ppJSClass, JSPropertySpec** ppJSProps, JSFunctionSpec** ppJSMethods, void** ppPrivate)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"GetJSClass called.");
	
	myclass.GetJSClass(ppJSClass, ppJSProps, ppJSMethods, ppPrivate);

	return TRUE;
}

// Called when the plugin should release its resources
// ---------------------------------------------------------------------------------
PLUG_API BOOL Destroy(DWORD instancehandle)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"Destroy called.");

	myclass.DocumentDestroy();
	
	return TRUE;
}

// Called when the DirectX device is lost (such as on a resize)
// ---------------------------------------------------------------------------------
PLUG_API BOOL DeviceLost(DWORD instancehandle)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"DeviceLost called.");
	
	myclass.DeviceLost();

	return TRUE;
}

// Called when the DirectX device is restored after a loss
// ---------------------------------------------------------------------------------
PLUG_API BOOL DeviceRestore(DWORD instancehandle)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"DeviceRestore called.");
	
	myclass.DeviceRestore();

	return TRUE;
}

// Called when the dxstudio document has finished loading
// ---------------------------------------------------------------------------------
PLUG_API BOOL DocumentLoaded(DWORD instancehandle)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"DocumentLoaded called.");
	
	myclass.DocumentLoaded();

	return TRUE;
}

// Called for every render stage and pass requested
// ---------------------------------------------------------------------------------
PLUG_API BOOL EngineStage(DWORD instancehandle, ES stage, int pass, SEngineInfo* ei)
// ---------------------------------------------------------------------------------
{
	/*GL_WriteDebugOut(L"EngineStage called.");
	
	switch(stage)
	{
	case ESRenderPrepare:		break;
	case ESRenderBegin:			break;
	case ESRenderEnd:			break;
	case ESRenderSceneBegin:	break;
	case ESRenderSceneEnd:		break;
	case ESRenderSceneObject:	break;
	case ESUpdateBegin:			break;
	case ESUpdateEnd:			break;
	case ESRenderSceneObjectInstance: break;
	}
	*/
	return TRUE;
}

// Called once per frame
// ---------------------------------------------------------------------------------
PLUG_API BOOL Update(DWORD instancehandle, FLOAT seconds)
// ---------------------------------------------------------------------------------
{
	//GL_WriteDebugOut(L"Update called.");

	myclass.Update(seconds);

	return TRUE;
}

// Utility function to write to the debug output
// ---------------------------------------------------------------------------------
void GL_WriteDebugOut(const WCHAR* str, ...)
// ---------------------------------------------------------------------------------
{
#ifdef _DEBUG
		
	static WCHAR buf1[20000];
	static WCHAR buf2[20000];

	va_list ptr;
	va_start(ptr,str);
	_vsnwprintf(buf1,19950,str,ptr);

	swprintf(buf2,20000,L"%s: %s",MY_DEBUG_NAME,buf1);

	OutputDebugString(buf2);

#endif
}

// Utility function to write to the console in the player
// ---------------------------------------------------------------------------------
void GL_WriteConsoleString(const WCHAR* str)
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"GL_WriteConsoleString called: ",str);
	
	EnterCriticalSection(g_pInfo->pJSCS); // Need this as only one thread is allowed JS access at a time
	JSObject* obj = JS_GetGlobalObject(g_pInfo->pJSContext); // We want the global object not the local one
	
	jsval argv, rval;
	argv = STRING_TO_JSVAL(JS_NewUCStringCopyN(g_pInfo->pJSContext, (const jschar*) str, wcslen(str)));
	
	JS_CallFunctionName(g_pInfo->pJSContext, obj, "print", 1, &argv, &rval);
	
	LeaveCriticalSection(g_pInfo->pJSCS);
}

// Utility function for memory checking (Debug)
// ---------------------------------------------------------------------------------
void GL_ActivateMemoryDebugging()
// ---------------------------------------------------------------------------------
{
	GL_WriteDebugOut(L"GL_ActivateMemoryDebugging called.");
	
	int iTmp = _CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
	
	// leak check at exit, check all allocations
	iTmp = (iTmp & 0x0000FFFF) | _CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_LEAK_CHECK_DF; 
	
	_CrtSetDbgFlag(iTmp);  
}

