//
// plugin.h : Header for standard plugin functions
//

#pragma once

typedef enum {ESRenderPrepare, ESRenderBegin, ESRenderEnd, ESRenderSceneBegin, ESRenderSceneEnd, ESRenderSceneObject, ESUpdateBegin, ESUpdateEnd, ESRenderSceneObjectInstance, NES} ES;

// ---------------------------------------------------------------------------------
typedef struct 
{
	LPDIRECT3D9 pDD;
	LPDIRECT3DDEVICE9 pDevice;
	D3DPRESENT_PARAMETERS d3dpp; 
	HWND hWnd;
	D3DSURFACE_DESC desc;
	D3DCAPS9 caps;
	const WCHAR* plugincachefolder;
	FLOAT* pTimer;
	JSContext* pJSContext;
	CRITICAL_SECTION* pJSCS;
	JSContext* pJSContextLocal;
	JSClass* pJSC_Vector;
	JSClass* pJSC_Rotation;
	JSClass* pJSC_Color;
} SPlayerInfo;
// ---------------------------------------------------------------------------------

#define RENDERFLAG_QUALITY (1<<0)

// ---------------------------------------------------------------------------------
typedef struct
{
	D3DXMATRIX mworld, mview, mproj;
	D3DXVECTOR3 camera_pos;
	D3DXQUATERNION camera_rot;
	LPDIRECT3DCUBETEXTURE9 cubetexture;
	FLOAT fov, aspectratio;
	DWORD rendertype;
	DWORD renderflags;
} SEngineInfo;
// ---------------------------------------------------------------------------------

#define SAFE_DELETE(p)       { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_DELETE_ARRAY(p) { if(p) { delete[] (p);   (p)=NULL; } }
#define SAFE_RELEASE(p)      { if(p) { (p)->Release(); (p)=NULL; } }
#define SAFE_CLOSEHANDLE(p)  { if(p) { CloseHandle(p); (p)=NULL; } }

// ---------------------------------------------------------------------------------
class CDXVector : public D3DXVECTOR3
{
public:
	CDXVector() : D3DXVECTOR3(), writeback(NULL), writebackflag(NULL) { }
	CDXVector(float x, float y, float z) : D3DXVECTOR3(x,y,z), writeback(NULL), writebackflag(NULL) { }
	CDXVector(const D3DVECTOR& v) : D3DXVECTOR3(v), writeback(NULL), writebackflag(NULL) { }
public:
	D3DXVECTOR3* writeback;
	BOOL* writebackflag;
};
// ---------------------------------------------------------------------------------
class CDXRotation : public D3DXQUATERNION
{
public:
	CDXRotation() : D3DXQUATERNION(), writeback(NULL), writebackflag(NULL) { }
	CDXRotation(float x, float y, float z, float w) : D3DXQUATERNION(x,y,z,w), writeback(NULL), writebackflag(NULL) { }
	CDXRotation(const D3DXQUATERNION& q) : D3DXQUATERNION(q), writeback(NULL), writebackflag(NULL) { }
public:
	D3DXQUATERNION* writeback;
	BOOL* writebackflag;
};
// ---------------------------------------------------------------------------------

#include "resource.h"		// main symbols

// Forward declarations
void GL_WriteDebugOut(const WCHAR* str, ...);
void GL_WriteConsoleString(const WCHAR* str);
void GL_ActivateMemoryDebugging(void);

extern SPlayerInfo* g_pInfo;
